C DILATOMETRIC ANALYSIS PROGRAM 4 September 1989
C H. K. D. H. Bhadeshia.  Oak Ridge Book, p.17
C------------------------------------------------
C  NEW VERSION, May 1999
C
C The non-linearity solution approach is enhanced
C Sree Harsha Lalam
C-------------------------------------------------
C  Typical Dataset
C
C  1 (No. of L-T pairs) C Si Mn Ni Mo Cr V wt.%
C  0.03 (Amount of C trapped in ferrite, wt.%) EXA EXG (expansivities
C                                                       of alph, gamm resp)
C  0.002 (relative length change) 415.0 (centigrade)
C   
C    End of typical dataset
C
C -------------------------------------------------------------------
C
C Lattice parameter of austenite calculated from Dyson, taking account
C of all alloying additions.
C
C If problems with Newton iteration, adjust SENS to lower value 
C
C Variation of austenite lattice parameter as a function of carbon
C 3.573+.033*XBAR  
C
C XML   carbon content of residual austenite, mole fraction
C XWT   carbon content of residual austenite, weight percent
C AA    ferrite lattice parameter at 298K
C GG    austenite lattice parameter at 298K
C AAT   ferrite lattice parameter at temperature
C AGT   carbon free austenite lattice parameter at temperature
C AGTX  carbon enriched austenite lattice parameter at temperature
C VA    is the volume fraction of ferrite
C T     is the temperature in Centigrade
C EXA   expansion coefficient of ferrite, 1/K
C EXG   expansion coefficient of austenite, 1/K
C PP    the lattice parameter of ferrite in Angstroms
C IPARAM set to zero to read in ferrite and austenite lattice parameters
C GG0   austenite lattice parameter at room temperature for pure iron, angstroms
C 
C ----------------------------------------------------------------
C
      IMPLICIT NONE 
      DOUBLE PRECISION A1,A11,A12,AA,AAT,AGT,AGTX,AV,CR,CVA,DFUNC
      DOUBLE PRECISION DVA,EXA,EXG,GG,GG0,MN,MO,NI,PFE,PP,RMCR
      DOUBLE PRECISION RMFE,RMMN,RMMO,RMNI,RMSI,RTEMP,RMVAN,SENS 
      DOUBLE PRECISION SI,VA,VA1,VA2,VAN,XALPHA,XBAR,XG,XGM
      DOUBLE PRECISION L(1000),T(1000),TIME(1000)
      INTEGER IALPH,IDATA,I,III,IPARAM
C
      OPEN(UNIT=3, FILE='LENGTH.DAT',STATUS='OLD')
      REWIND 3
      OPEN(UNIT=4, FILE='OUT.DAT')
      REWIND 4
C
C Physical constants used in calculations.
C Reference : "Fundamental physical constants and energy conversion factors",
C National Physical Laboratory.
C
C
C  AV is the Avogadro constant (in 1/(mole)
C
      AV = 6.0221367D+23
C
C Set room temperature
C
      RTEMP = 25.0D+00
C
C Set ferrite lattice parameter at 298K
C
      PFE = 2.8664D-10
C
C SET sensitivity
C
      SENS=0.1D-01
C
      READ(3,*) IDATA,XBAR,SI,MN,NI,MO,CR,VAN
      READ(3,*) XALPHA,EXA,EXG,IPARAM
C
      IF (IPARAM .EQ. 0) THEN
         GG0 = 3.573D-10
         CALL MAP_STEEL_FERR(XBAR,SI,MN,NI,MO,CR,VAN,XALPHA,PFE,PP)
      ELSE
         READ(3,*) GG0,PP
      ENDIF
C
      WRITE(4,1) XALPHA,EXA,EXG,RTEMP,PP
      WRITE (4,2) XBAR,MN,SI,NI,CR,MO,VAN
C
      DO 100 I=1,IDATA
         READ (3,*) L(I),T(I),TIME(I)
  100 CONTINUE
      GG = 0.00095*MN - 0.0002*NI + 0.0006*CR +
     &     0.0031*MO + 0.0018*VAN + GG0*1.0D10 + 0.033*XBAR     
      AA = PP*1.0D10
      DO 170 IALPH=1,1
C
C Initial Approximation of ferrite volume
C
         VA = 0.7D00
         DO 160 I=1,IDATA
	    III = 0
	    WRITE (*,*) 'Data No. = ',I
            AAT = AA*(1.0D+00 + (T(I)-RTEMP)*EXA)
            AGT = GG*(1.0D+00 + (T(I)-RTEMP)*EXG)
	    VA = 0.7D00
C
C ****************************************************
C Modified by Harsha
C ****************************************************

  120       CONTINUE
            CALL MAP_STEEL_PARAM2(AGTX,AGT,AAT,VA,XBAR,XALPHA,
     &                        XG,EXG,T(I),AV,RTEMP)
C 
C CALCULATING THE DERIVATIVE OF THE FUNCTION
C 
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA,L(I),A1)
	    IF (DABS(A1) .GT. 0.1D-05) GOTO 130
	    GOTO 150
  130       DVA = 0.01*VA	
	    CVA = DABS(VA)
	    IF (CVA .LE. 0.01) THEN
	    DVA = 0.01
	    ENDIF
	    VA1 = VA + DVA
            CALL MAP_STEEL_PARAM2(AGTX,AGT,AAT,VA1,XBAR,XALPHA,
     &                        XG,EXG,T(I),AV,RTEMP)	
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA1,L(I),A12)
	    VA2 = VA - DVA
            CALL MAP_STEEL_PARAM2(AGTX,AGT,AAT,VA2,XBAR,XALPHA,
     &                         XG,EXG,T(I),AV,RTEMP)	
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA2,L(I),A11)
	    DFUNC = (A12-A11)/(2.0*DVA)
	    VAN = VA - SENS*(A1/DFUNC)	
            III = III + 1
C
C           WRITE(*,*) 'Data No. = ',I,',  No. of Intreations= ',III
C
            IF (III .GT. 10000) THEN
               WRITE(*,3)
               STOP
            ENDIF
	    VA = VAN
  140       IF (VA .GT. 0.99) THEN
	       VA = 0.8*VA
	       GOTO 140
	    ENDIF
	    IF (VA .LE. 0.0001) THEN
	       VA = 1.2*VA
               GOTO 140
	    ENDIF		
	    GOTO 120
C
C ****************************************************
C End of modifications
C ****************************************************
C
  150       CALL MAP_STEEL_ATM(XG,XGM,XBAR,SI,MN,NI,MO,CR,VAN,
     &           RMSI,RMMN,RMNI,RMMO,RMCR,RMVAN,RMFE)
            WRITE(4,4) T(I),L(I),TIME(I),VA,XG,XGM,AAT,AGT,AGTX,AA
  160    CONTINUE
  170 CONTINUE
      STOP
    1 FORMAT(' Amount of carbon trapped in ferrite, wt.% =',F8.4/
     &       '          Linear expansivity of alpha, 1/K =',D12.5/
     &       '          Linear expansivity of gamma, 1/K =',D12.5/
     &       '          Ambient temperature, Centigrade  =',F8.0/
     &       '          Calculated ferrite parameter, m  =',D12.4)
    2 FORMAT (/' STEEL COMPOSITION, wt.% '/
     &   7H Carbon,F10.3/10H Manganese,F7.3,10X,8H Silicon,F9.3,
     &   10X,7H Nickel,F10.3/9H Chromium,F8.3,10X,11H Molybdenum, F6.3,
     &   10X,9H Vanadium,F8.3//
     &   ' Ctemp   Del-L  Time(s) V-alpha  Xg-wt%    Xg-Mfrc ',
     &   ' AAT Ang   AGT Ang  AGTX Ang   AA'/)
    3 FORMAT(' *****************************'///
     &       '  TOO MANY ITERATION STEPS, Check DATA or SENS'///
     &       ' ******************************************')
    4 FORMAT(F6.0,F8.5,F6.1,F8.3,2F10.4,4F10.5)
      END
C	
C -----------------------------------------------------------------
C
C converts wt.% carbon in enriched austenite, to mole fraction,
C given the alloy composition prior to enrichment, and the wt.%
C carbon following enrichment.
C
      SUBROUTINE MAP_STEEL_ATM(XG,XGM,XBAR,DSI,DMN,DNI,DMO,DCR,
     & DVAN,MSI,MMN,MNI,MMO,MCR,MVAN,MFE)
      IMPLICIT NONE                           
      DOUBLE PRECISION XG,XGM,XBAR,DSI,DMN,DNI,DMO,DCR,DVAN,
     &MSI,MMN,MNI,MMO,MCR,MVAN,MFE,M,BSI,BMN,BNI,BCR,BMO,BVAN,
     &BFE,BC,C,SI,MN,NI,MO,CR,VAN,ASI,AMN,ANI,ACR,AMO,AVAN,
     &AFE,AC,Z,ZSI,ZMN,ZNI,ZMO,ZCR,ZVAN,ZFE,FE
      INTEGER I
C
        I=0
C
C atomic weights
C
        BSI=28.0855
        BMN=54.938
        BNI=58.71
        BCR=51.996
        BMO=95.94
        BVAN=50.94
        BFE=55.847
        BC=12.011
C
C
        C=XBAR
        SI=DSI
        MN=DMN
        NI=DNI
        MO=DMO
        CR=DCR
        VAN=DVAN
C
C Divide wt.% by atomic weights
C
 1       ASI=SI/BSI    
        AMN=MN/BMN
        ANI=NI/BNI
        ACR=CR/BCR
        AMO=MO/BMO
        AVAN=VAN/BVAN
        AFE=FE/BFE
        AC=C/BC
C
C Calculate susbtitutional atom fractions
C
      Z=ASI+AMN+ANI+ACR+AMO+AVAN+AFE
      M=Z+AC
        ZSI=ASI/Z
        ZMN=AMN/Z
        ZNI=ANI/Z
        ZCR=ACR/Z
        ZMO=AMO/Z
        ZVAN=AVAN/Z
        ZFE=AFE/Z
C
        IF (I .GT. 0) GOTO 3
C
C new values allowing for carbon enrichment
C
        SI=ZSI*BSI*(1.0D+02-XG)/BFE
        MN=ZMN*BMN*(1.0D+02-XG)/BFE
        NI=ZNI*BNI*(1.0D+02-XG)/BFE
        MO=ZMO*BMO*(1.0D+02-XG)/BFE
        CR=ZCR*BCR*(1.0D+02-XG)/BFE
        VAN=ZVAN*BVAN*(1.0D+02-XG)/BFE
        FE=ZFE*BFE*(1.0D+02-XG)/BFE
C
      I=2
      C=XG
      GOTO 1
C
C new values of mole fractions
C
3       MSI=ASI/M
        MMN=AMN/M
        MNI=ANI/M
        MCR=ACR/M
        MMO=AMO/M
        MVAN=AVAN/M
C       MCO=ACO/M
        MFE=AFE/M
      XGM=AC/M
C
        RETURN
        END
C
C -----------------------------------------------------------------
C Calculates the lattice parameter of enriched austenite, after a
C degree of transformation to ferrite.
C
      SUBROUTINE MAP_STEEL_PARAM2(AGTX,AGT,AAT,VA,XBAR,XALPHA,
     &                             XG,EXG,T,AV,RTEMP)
C
      IMPLICIT NONE
      DOUBLE PRECISION AAT,AGES,AGT,AGTX,AGTX1,AGTX2,AV,DIFF
      DOUBLE PRECISION EXG,RHOA,RHOG,RTEMP,T,VA
      DOUBLE PRECISION XALPHA,XBAR,XG
      INTEGER II
C
C  Guess a value AGES of AGTX and then iterate 
C  
      AGTX = AGT
      AGES = AGTX
C
C************************************************************
C Modified by Harsha
C************************************************************
C
      AGTX2 = AGT
  100 AGTX1 = AGTX2
C	
C A2 is the mole fraction of iron atoms in ferrite
C Make the approximation that density Rho depends only on atomic wt. of iron
C See p.58, Oak Ridge Book
C
      RHOA = 55.847*2.0/(AV*AAT*AAT*AAT*1.0D-30)
      RHOG = 55.847*4.0/(AV*AGTX1*AGTX1*AGTX1*1.0D-30)
      XG   = (VA*RHOA*(XBAR-XALPHA) + XBAR*RHOG*(1.0-VA))
     &        /((1.0-VA)*RHOG)
C
C XG is the wt% Carbon in the enriched austenite. 
C Note XBAR and XALPHA are in wt%.
C
C calculate new value of enriched austenite lattice parameter 
C
      AGTX2 = AGT+0.033*(XG-XBAR)*(1.0D+00 + (T-RTEMP)*EXG) 
      DIFF  = (AGTX1-AGTX2)/AGTX1
      DIFF  = DABS(DIFF)
C
C Check for Convergence of AGTX
C
      II = II+1
      IF (II .GT. 10D+10) THEN
         WRITE(*,1)
         STOP
      ENDIF
      IF (DIFF .GT. 1.0D-10) GOTO 100
      AGTX = AGTX2
      RETURN
    1 FORMAT(' *****************************'///
     &       '  TOO MANY ITERATION STEPS, Check DATA '///
     &       ' ******************************************')
      END
C
C ----------------------------------------------------------------
      SUBROUTINE MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA,DL,A1)
C
      IMPLICIT NONE                          
      DOUBLE PRECISION AGTX,AGT,AAT,VA,DL,A1,A,A2
C
      A  = VA/(1.0D0-VA)                   
      A2 = 1.0D0/(1.0D0 + (2.0D0/A)*(AAT**3/AGTX**3))
      A1 = 1.0D0-3.0D0*DL-((AGT**3)/
     &         ((AGTX**3) + (2.0D0*(AAT**3)-(AGTX**3))*A2))
      RETURN
      END
C-----------------------------------------------------------------
C
C Calculate ferrite lattice parameter P at room temperature for alloy steel
C Data from Leslie, concentrations input as wt.%, balance iron
C Mo effect F. Wever, Zeitschrift fur Metallkunde 20 (1928) 363-370
C CALP is wt.% carbon dissolved in ferrite
C Vanadium coefficient from Hume-Rothery an Sutton (Structure of Iron+Alloys)
C Set PFE as the parameter of pure iron at room temperature
C Value of 2.8664 Ansgtroms for PFE from National Bureau of Standards (USA)
C Circ. 539, 4 3 (1955)
C H. K. D. H. Bhadeshia, 6 September 1989
C
      SUBROUTINE MAP_STEEL_FERR(C,SI,MN,NI,MO,CR,V,CALP,PFE,P)
      IMPLICIT NONE 
      DOUBLE PRECISION C,SI,MN,NI,MO,CR,V,CALP,P,PFE,ASI,AMN,
     &ANI,ACR,AMO,AV,FE,AFE,AC,M,MSI,MMN,MNI,MCR,MMO,MFE,MC,
     &MV,V1,V2,CC
C
      ASI=SI/28.0855
      AMN=MN/54.938
      ANI=NI/58.71
      ACR=CR/51.996
      AMO=MO/95.94
      AV=V/50.9415
C     AW=W/183.85
C     ACO=CO/58.9332
C
      FE=100.0-C-SI-MN-NI-MO-CR-V
      AFE=FE/55.847
      AC=C/12.011
C
      M=AC+ASI+AMN+ANI+AMO+ACR+AV+AFE
      MSI=ASI/M
      MMN=AMN/M
      MNI=ANI/M
      MCR=ACR/M
      MMO=AMO/M
C     MW=AW/M
C     MCO=ACO/M
      MFE=AFE/M
      MC=AC/M
      MV=AV/M
C
C expansion caused by carbon 
C Reference: C. S. Roberts, Trans. AIME, 197 (1953) 203
C
      V1=PFE*PFE*PFE
      V2=(PFE - 0.013D-10*CALP)*(PFE - 0.013D-10*CALP)*
     &   (PFE + 0.116D-10*CALP)
      CC=(V2-V1)/(3.0*PFE*PFE)
C
      P=PFE+CC - MSI*3.0D-12 + MMN*0.06D-10 + MNI*0.07D-10
     &  + MMO*0.31D-10 + MCR*0.05D-10 + MV*0.096D-10 
      RETURN
      END

